import java.util.Collection;


/**
 * <p>Represents the model for a chat server, storing information about users
 * and channels on the server.</p>
 *
 * <p>The {@link ServerBackend} interacts with this class via the {@link
 * #registerUser(int)}, {@link #deregisterUser(int)}, and {@link #handle(int,
 * Command)} methods. Each of these methods generates a {@link Broadcast}
 * representing the set of commands that should be sent to other clients
 * connected to the server as a result of the interaction. See the <a
 * href="../spec.shtml">protocol specification</a> for an overview of the
 * {@code ServerModel}'s role in the program.</p>
 *
 * <p>You may add {@code private} fields and helper methods to this file if you
 * wish, but you may <i>not</i> modify any {@code public} method names or
 * signatures.</p>
 */
public final class ServerModel {
    
    /**
     * <p>Constructs a {@code ServerModel} with any collections needed for
     * modeling the server state initialized.</p>
     */
    public ServerModel() {
        // TODO: initialize your state here
    }

    /**
     * Informs the model that a client has connected to the server with the
     * given user ID. The model should update its state so that it can
     * identify this user during later interactions with the model. Any user
     * that is registered with the server (without being later deregistered)
     * should appear in the output of {@link #getRegisteredUsers()}.
     *
     * @param userId
     *         the unique user ID created by the backend to represent the user.
     * @return broadcast representing messages that should be sent as a a result
     * of registering the user
     */
    public Broadcast registerUser(int userId) {
        // TODO: register users when they connect
        return null;
    }

    /**
     * <p>Informs the model that the client with the given user ID has
     * disconnected from the server. After a user ID is deregisted, the server
     * backend is free to re-assign this user ID to an entirely different
     * client. As such, the model should take care to expunge any state
     * pertaining to a user who is deregistered. Any user that is deregistered
     * (without later being registered) should not appear in the output of
     * {@link #getRegisteredUsers()}.</p>
     *
     * <p>The behavior of this method if the given user ID is not registered
     * with the model is undefined.</p>
     *
     * @param userId
     *         the unique user ID created by the backend to represent the user.
     * @return broadcast representing messages that should be sent as a a result
     * of deregistering the user
     */
    public Broadcast deregisterUser(int userId) {
        // TODO: deregister users when they disconnect
        return null;
    }

    /**
     * <p>Passes a command along to the model for processing. In response, the
     * model should process the command, updating its internal state and
     * generating a {@link Broadcast} representing any messages that should
     * be sent to clients in response to the message. The exact behavior is
     * dependent on the internal state of the model and the exact nature of the
     * input command; these behaviors are specified in the protocol
     * specification.</p>
     *
     * <p>The behavior of this method if the given user ID is not registered
     * with the model is undefined.</p>
     *
     * @param userId
     *         the unique user ID created by the backend to represent the user.
     * @param command
     *         the command to be processed by the model
     * @return broadcast representing messages that should be sent as as a
     * result of {@code command}
     */
    public Broadcast handle(int userId, Command command) {
        // TODO: handle commands, update state
        return null;
    }

    /**
     * Returns a collection of the nicknames of all users that are registered
     * with the server. Provided for testing.
     *
     * @return the collection of registered user nicknames
     */
    public Collection<String> getRegisteredUsers() {
        // TODO: return registered users
        return null;
    }

    /**
     * Returns a collection of the names of all channels that are present on
     * the server. Provided for testing.
     *
     * @return the collection of channel names
     */
    public Collection<String> getChannels() {
        // TODO: return channels
        return null;
    }

    /**
     * Returns a collection of the nicknames of all users in a given channel.
     * The returned collection is empty if no channel with the given channel
     * name exists. Provided for testing.
     *
     * @param channelName
     *         the channel whose member nicknames should be returned
     * @return the collection of user names on the specified channel
     */
    public Collection<String> getUsers(String channelName) {
        // TODO: return the users on the channel
        return null;
    }

    /**
     * Returns the nickname of the owner of the given channel. The result is
     * {@code null} if no channel with the given channel name exists.
     * Provided for testing.
     *
     * @param channelName
     *         the channel whose owner nickname should be returned
     * @return the nickname of the channel owner
     */
    public String getOwner(String channelName) {
        // TODO: return the owner of the channel
        return null;
    }
    
}
